#include "nrf.h"

#define DIR_POS     ((uint32_t) 0)
#define DIR_INPUT   ((uint32_t) 0x00) // Set pin as input
#define DIR_OUTPUT  ((uint32_t) 0x01) // Set pin as output

#define INPUT_POS   ((uint32_t) 1)
#define INPUT_CONN  ((uint32_t) 0x00) // Connect input buffer
#define INPUT_DISC  ((uint32_t) 0x01) // Disconnect input buffer

#define PULL_POS    ((uint32_t) 2)
#define PULL_NONE   ((uint32_t) 0x00) // No pullup/down resistor on pin
#define PULL_DOWN   ((uint32_t) 0x01) // Set pull down resistor on pin
#define PULL_UP     ((uint32_t) 0x03) // Set pull up resistor on pin

#define DRIVE_POS   ((uint32_t) 8)
#define DRIVE_S0S1  ((uint32_t) 0x00) 
#define DRIVE_H0S1  ((uint32_t) 0x01) 
#define DRIVE_S0H1  ((uint32_t) 0x02) 
#define DRIVE_H0H1  ((uint32_t) 0x03) 
#define DRIVE_D0S1  ((uint32_t) 0x04) 

#define SENSE_POS   ((uint32_t) 16)
#define SENSE_DIS   ((uint32_t) 0x00) //Disable sense
#define SENSE_HI    ((uint32_t) 0x02) //High sense
#define SENSE_LO    ((uint32_t) 0x03) //Low sense


#define MODE_POS    ((uint32_t) 0)
#define MODE_DIS    ((uint32_t) 0x00) // disable gpiote modes
#define MODE_EVT    ((uint32_t) 0x01) // enable gpiote event
#define MODE_TSK    ((uint32_t) 0x03) // enable gpiote task event

#define PSEL_POS    ((uint32_t) 8)

#define POLARITY_POS  ((uint32_t) 16)
#define POLARITY_NONE ((uint32_t) 0x00)
#define POLARITY_HTL  ((uint32_t) 0x02)
#define POLARITY_LTH  ((uint32_t) 0x01)
#define POLARITY_TGL  ((uint32_t) 0x03)

#define OUTINIT_POS   ((uint32_t) 20)
#define OUTINIT_LOW   ((uint32_t) 0x00)
#define OUTINIT_HIGH  ((uint32_t) 0x01)

#define MAX_NUM_CHANNELS 8

typedef void (*gpiote_event_handler_t) (void);

typedef struct
{
  NRF_GPIO_Type *port;
  uint32_t      pin;
  uint32_t      dir;
  uint32_t      input_buffer;
  uint32_t      pull;
  uint32_t      drive;
  uint32_t      sense;
}gpio_config;

typedef struct
{
  NRF_GPIOTE_Type *port;
  uint32_t mode;
  uint32_t pin;
  uint32_t polarity;
  uint32_t outinit;
  uint32_t channel_num;
  gpiote_event_handler_t event_handler;
}gpiote_config;


void gpio_config_pin(gpio_config *config);

void gpio_pin_set(NRF_GPIO_Type *port, uint32_t pin_num);

void gpio_pin_clear(NRF_GPIO_Type *port, uint32_t pin_num);

void gpiote_enable_interrupt(uint32_t channel_num);

void gpiote_disable_interrupt(uint32_t channel_num);

void gpiote_config_channel(gpiote_config *config);

void gpiote_interrupt_init(void);