#include "nrf.h"

#define DIR_POS     ((uint32_t) 0)
#define DIR_INPUT   ((uint32_t) 0x00) // Set pin as input
#define DIR_OUTPUT  ((uint32_t) 0x01) // Set pin as output

#define TIMER_SHORT_CMP0_CLR_POS ((uint32_t) 0)
#define TIMER_SHORT_CMP0_CLR_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP0_CLR_DISABLE ((uint32_t) 0)

#define TIMER_SHORT_CMP1_CLR_POS ((uint32_t) 1)
#define TIMER_SHORT_CMP1_CLR_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP1_CLR_DISABLE ((uint32_t) 0)

#define TIMER_SHORT_CMP2_CLR_POS ((uint32_t) 2)
#define TIMER_SHORT_CMP2_CLR_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP2_CLR_DISABLE ((uint32_t) 0)

#define TIMER_SHORT_CMP3_CLR_POS ((uint32_t) 3)
#define TIMER_SHORT_CMP3_CLR_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP3_CLR_DISABLE ((uint32_t) 0)

#define TIMER_SHORT_CMP4_CLR_POS ((uint32_t) 4)
#define TIMER_SHORT_CMP4_CLR_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP4_CLR_DISABLE ((uint32_t) 0)

#define TIMER_SHORT_CMP5_CLR_POS ((uint32_t) 5)
#define TIMER_SHORT_CMP5_CLR_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP5_CLR_DISABLE ((uint32_t) 0)

#define TIMER_SHORT_CMP0_STP_POS ((uint32_t) 8)
#define TIMER_SHORT_CMP0_STP_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP0_STP_DISABLE ((uint32_t) 0)

#define TIMER_SHORT_CMP1_STP_POS ((uint32_t) 9)
#define TIMER_SHORT_CMP1_STP_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP1_STP_DISABLE ((uint32_t) 0)

#define TIMER_SHORT_CMP2_STP_POS ((uint32_t) 10)
#define TIMER_SHORT_CMP2_STP_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP2_STP_DISABLE ((uint32_t) 0)

#define TIMER_SHORT_CMP3_STP_POS ((uint32_t) 11)
#define TIMER_SHORT_CMP3_STP_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP3_STP_DISABLE ((uint32_t) 0)

#define TIMER_SHORT_CMP4_STP_POS ((uint32_t) 12)
#define TIMER_SHORT_CMP4_STP_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP4_STP_DISABLE ((uint32_t) 0)

#define TIMER_SHORT_CMP5_STP_POS ((uint32_t) 13)
#define TIMER_SHORT_CMP5_STP_ENABLE ((uint32_t) 1)
#define TIMER_SHORT_CMP5_STP_DISABLE ((uint32_t) 0)

#define TIMER_INT_CMP0_POS ((uint32_t) 16)
#define TIMER_INT_CMP1_POS ((uint32_t) 17)
#define TIMER_INT_CMP2_POS ((uint32_t) 18)
#define TIMER_INT_CMP3_POS ((uint32_t) 19)
#define TIMER_INT_CMP4_POS ((uint32_t) 20)
#define TIMER_INT_CMP5_POS ((uint32_t) 21)

#define TIMER_MODE_TIMER   ((uint32_t) 0x00)
#define TIMER_MODE_LPCMP   ((uint32_t) 0x02)

#define TIMER_BITMODE_16   ((uint32_t) 0x00)
#define TIMER_BITMODE_8    ((uint32_t) 0x01)
#define TIMER_BITMODE_24   ((uint32_t) 0x02)
#define TIMER_BITMODE_32   ((uint32_t) 0x03)

#define TIMER_PRESCALER_0  ((uint32_t) 0x00)
#define TIMER_PRESCALER_1  ((uint32_t) 0x01)
#define TIMER_PRESCALER_2  ((uint32_t) 0x02)
#define TIMER_PRESCALER_3  ((uint32_t) 0x03)
#define TIMER_PRESCALER_4  ((uint32_t) 0x04)
#define TIMER_PRESCALER_5  ((uint32_t) 0x05)
#define TIMER_PRESCALER_6  ((uint32_t) 0x06)
#define TIMER_PRESCALER_7  ((uint32_t) 0x07)
#define TIMER_PRESCALER_8  ((uint32_t) 0x08)
#define TIMER_PRESCALER_9  ((uint32_t) 0x09)

#define MAX_TIMER_INSTANCES 4
#define MAX_CHANNEL_INSTANCES 4


typedef void (*timer_event_handler_t) (void);

//typedef struct
//{
//  NRF_GPIO_Type *port;
//  uint32_t      pin;
//  uint32_t      mode;
//  uint32_t      input_buffer;
//  uint32_t      pull;
//  uint32_t      drive;
//  uint32_t      sense;
//}gpio_config;

//
//typedef struct
//{
//  NRF_GPIOTE_Type *port;
//  uint32_t mode;
//  uint32_t pin;
//  uint32_t polarity;
//  uint32_t outinit;
//  uint32_t channel_num;
//  gpiote_event_handler_t event_handler;
//}gpiote_config;


typedef struct
{
  uint32_t event_compare;
  timer_event_handler_t event_handler;
}timer_callback;

void enable_timer_instance(NRF_TIMER_Type *instance);

void set_timer_shorts(NRF_TIMER_Type *instance, uint32_t shorts_pos, uint32_t sel_short);
//
void set_timer_interrupt(NRF_TIMER_Type *instance, uint32_t int_pos, uint8_t enable);

void set_timer_mode(NRF_TIMER_Type *instance,uint32_t mode);

void set_timer_bitmode(NRF_TIMER_Type *instance, uint32_t mode);

void set_timer_prescaler(NRF_TIMER_Type *instance, uint32_t prescaler);

void set_timer_compare(NRF_TIMER_Type *instance, uint32_t channel, uint32_t compare_val, timer_event_handler_t event_handler);

void stop_timer(NRF_TIMER_Type *instance);

void start_timer(NRF_TIMER_Type *instance);

void clear_timer(NRF_TIMER_Type *instance);

void capture_timer(NRF_TIMER_Type *instance, uint32_t channel);
