#include "gpio.h"

//Array for holding custom event handlers
gpiote_event_handler_t event_handlers[MAX_NUM_CHANNELS];


//Interrupt handler for gpiote peripheral
void GPIOTE_IRQHandler(void)
{
  //Goes through each event handler if it is valid
  for(int x = 0; x < MAX_NUM_CHANNELS; x++)
  {
    if(event_handlers[x] != 0)
    {
      event_handlers[x]();
    }
  }
}

// Configure pin for gpio operations
void gpio_config_pin(gpio_config *config)
{
  
  config->port->PIN_CNF[config->pin] = ((config->dir << DIR_POS)
                                       |(config->input_buffer << INPUT_POS)
                                       |(config->pull << PULL_POS)
                                       |(config->drive << DRIVE_POS)
                                       |(config->sense << SENSE_POS));
}

// Set the output pin high
void gpio_pin_set(NRF_GPIO_Type *port, uint32_t pin_num)
{
  port->OUTSET |= (1 << pin_num);
}

// Set the output pin low
void gpio_pin_clear(NRF_GPIO_Type *port, uint32_t pin_num)
{
  port-> OUTCLR |= (1 << pin_num);
}

// Enable interrupts for specified gpiote channel
void gpiote_enable_interrupt(uint32_t channel_num)
{
  NRF_GPIOTE->INTENSET |= (1 << channel_num);
}

// Disable interrupts for specified gpiote channel
void gpiote_disable_interrupt(uint32_t channel_num)
{
  NRF_GPIOTE->INTENCLR &= ~(1 << channel_num);
}

// Configure gpiote channel
void gpiote_config_channel(gpiote_config *config)
{
  //NRF_GPIOTE->CONFIG[config->channel_num] |= ((config->mode << MODE_POS)
  if(NRF_GPIOTE == config->port)
  {
    volatile y = 1;
    y++;
  }
  config->port->CONFIG[config->channel_num] |= ((config->mode << MODE_POS)
                                              |(config->pin << PSEL_POS)
                                              |(config->polarity << POLARITY_POS)
                                              |(config->outinit << OUTINIT_POS));
  event_handlers[config->channel_num] = config->event_handler;
  volatile uint32_t x = 312;
  x++;

}

// Enables use of GPIOTE_IRQHandler
void gpiote_interrupt_init(void)
{
  NVIC_EnableIRQ(GPIOTE_IRQn);
}